@@ -20,6 +20,7 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Martin Barisits <martin.barisits@cern.ch>, 2019
+# - Luc Goossens <luc.goossens@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -58,7 +59,8 @@ def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=No
     if rse is not None:
         rse_id = get_rse_id(rse=rse)
 
-    replicas = replica.list_bad_replicas_status(state=state, rse_id=rse_id, younger_than=younger_than, older_than=older_than, limit=limit, list_pfns=list_pfns)
+    replicas = replica.list_bad_replicas_status(state=state, rse_id=rse_id, younger_than=younger_than,
+                                                older_than=older_than, limit=limit, list_pfns=list_pfns)
     return [api_update_return_dict(r) for r in replicas]
 
 
@@ -133,7 +135,9 @@ def get_did_from_pfns(pfns, rse):
 def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
                   ignore_availability=True, all_states=False, rse_expression=None,
                   client_location=None, domain=None, signature_lifetime=None,
-                  resolve_archives=True, resolve_parents=False, issuer=None):
+                  resolve_archives=True, resolve_parents=False,
+                  updated_after=None,
+                  issuer=None):
     """
     List file replicas for a list of data identifiers.
 
@@ -148,6 +152,7 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
     :param signature_lifetime: If supported, in seconds, restrict the lifetime of the signed PFN.
     :param resolve_archives: When set to True, find archives which contain the replicas.
     :param resolve_parents: When set to True, find all parent datasets which contain the replicas.
+    :param updated_after: datetime object (UTC time), only return replicas updated after this time
     :param issuer: The issuer account.
     """
     validate_schema(name='r_dids', obj=dids)
@@ -167,7 +172,8 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
                                      all_states=all_states, rse_expression=rse_expression,
                                      client_location=client_location, domain=domain,
                                      sign_urls=sign_urls, signature_lifetime=signature_lifetime,
-                                     resolve_archives=resolve_archives, resolve_parents=resolve_parents)
+                                     resolve_archives=resolve_archives, resolve_parents=resolve_parents,
+                                     updated_after=updated_after)
 
     for rep in replicas:
         # 'rses' and 'states' use rse_id as the key. This needs updating to be rse.
@@ -251,7 +257,6 @@ def delete_replicas(rse, files, issuer, ignore_availability=False):
 
 
 def update_replicas_states(rse, files, issuer):
-
     """
     Update File replica information and state.
 
@@ -299,7 +304,7 @@ def list_dataset_replicas_vp(scope, name, deep=False):
     :param name: The name of the dataset.
     :param deep: Lookup at the file level.
 
-    :returns: If VP exists a list of dicts of sites, otherwise a list of dicts of dataset replicas
+    :returns: If VP exists a list of dicts of sites, otherwise nothing
 
     NOTICE: This is an RnD function and might change or go away at any time.
     """
