@@ -16,21 +16,25 @@
 # - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2018
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2019
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2017
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2019
 # - Frank Berghaus, <frank.berghaus@cern.ch>, 2017
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
 # - Tobias Wegner <twegner@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Gabriele Fronze' <gfronze@cern.ch>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@gmail.com>, 2019
 #
 # PY3K COMPATIBLE
 
+
 from __future__ import print_function
 
 import base64
+import copy
 import datetime
 import errno
 import getpass
@@ -51,7 +55,7 @@ import zlib
 from logging import getLogger, Formatter
 from logging.handlers import RotatingFileHandler
 from uuid import uuid4 as uuid
-from six import string_types
+from six import string_types, PY3
 from xml.etree import ElementTree
 
 try:
@@ -80,7 +84,7 @@ except ImportError:
     import urllib.parse as urlparse
 
 from rucio.common.config import config_get
-from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError
+from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError, RucioException
 from rucio.common.types import InternalAccount, InternalScope
 # delay import until function to avoid circular dependancy (note here for reference)
 # from rucio.core.rse import get_rse_name
@@ -153,9 +157,8 @@ def build_url(url, path=None, params=None, doseq=False):
     separated by '&' are generated for each element of the value sequence for the key.
     """
     complete_url = url
-    complete_url += "/"
     if path is not None:
-        complete_url += path
+        complete_url += "/" + path
     if params is not None:
         complete_url += "?"
         if isinstance(params, str):
@@ -165,6 +168,91 @@ def build_url(url, path=None, params=None, doseq=False):
     return complete_url
 
 
+def oidc_identity_string(sub, iss):
+    """
+    Transform IdP sub claim and issuers url into users identity string.
+    :param sub: users SUB claim from the Identity Provider
+    :param iss: issuer (IdP) https url
+
+    :returns: OIDC identity string "SUB=<usersid>, ISS=https://iam-test.ch/"
+    """
+    return 'SUB=' + str(sub) + ', ISS=' + str(iss)
+
+
+def sqlalchemy_obj_to_dict(sqlalchemyresult):
+    """
+    Makes dictionary from SQLAlchemy query result object
+    :param sqlalchemyresult:
+    :returns: dictionary
+    """
+    res_dict = copy.deepcopy(dict(sqlalchemyresult.__dict__))
+    del res_dict['_sa_instance_state']
+    return res_dict
+
+
+def all_oidc_req_claims_present(scope, audience, required_scope, required_audience, sepatator=" "):
+    """
+    Checks if both of the following statements are true:
+    - all items in required_scope are present in scope string
+    - all items in required_audience are present in audience
+    returns false otherwise. audience and scope must be both strings
+    or both lists. Similarly for required_* variables.
+    If this condition is satisfied, False is returned.
+    :params scope: list of strings or one string where items are separated by a separator input variable
+    :params audience: list of strings or one string where items are separated by a separator input variable
+    :params required_scope: list of strings or one string where items are separated by a separator input variable
+    :params required_audience: list of strings or one string where items are separated by a separator input variable
+    :params sepatator: separator string, space by default
+    :returns : True or False
+    """
+    if not scope:
+        scope = ""
+    if not audience:
+        audience = ""
+    if not required_scope:
+        required_scope = ""
+    if not required_audience:
+        required_audience = ""
+    if (isinstance(scope, list) and isinstance(audience, list) and  # NOQA: W504
+        isinstance(required_scope, list) and isinstance(required_audience, list)):
+        scope = [str(it) for it in scope]
+        audience = [str(it) for it in audience]
+        required_scope = [str(it) for it in required_scope]
+        required_audience = [str(it) for it in required_audience]
+        req_scope_present = all(elem in scope for elem in required_scope)
+        req_audience_present = all(elem in audience for elem in required_audience)
+        return req_scope_present and req_audience_present
+    elif (isinstance(scope, string_types) and isinstance(audience, string_types) and  # NOQA: W504
+          isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
+        scope = str(scope)
+        audience = str(audience)
+        required_scope = str(required_scope)
+        required_audience = str(required_audience)
+        req_scope_present = all(elem in scope.split(sepatator) for elem in required_scope.split(sepatator))
+        req_audience_present = all(elem in audience.split(sepatator) for elem in required_audience.split(sepatator))
+        return req_scope_present and req_audience_present
+    elif (isinstance(scope, list) and isinstance(audience, list) and  # NOQA: W504
+          isinstance(required_scope, string_types) and isinstance(required_audience, string_types)):
+        scope = [str(it) for it in scope]
+        audience = [str(it) for it in audience]
+        required_scope = str(required_scope)
+        required_audience = str(required_audience)
+        req_scope_present = all(elem in scope for elem in required_scope.split(sepatator))
+        req_audience_present = all(elem in audience for elem in required_audience.split(sepatator))
+        return req_scope_present and req_audience_present
+    elif (isinstance(scope, string_types) and isinstance(audience, string_types) and  # NOQA: W504
+          isinstance(required_scope, list) and isinstance(required_audience, list)):
+        scope = str(scope)
+        audience = str(audience)
+        required_scope = [str(it) for it in required_scope]
+        required_audience = [str(it) for it in required_audience]
+        req_scope_present = all(elem in scope.split(sepatator) for elem in required_scope)
+        req_audience_present = all(elem in audience.split(sepatator) for elem in required_audience)
+        return req_scope_present and req_audience_present
+    else:
+        return False
+
+
 def generate_uuid():
     return str(uuid()).replace('-', '').lower()
 
@@ -180,10 +268,38 @@ def clean_headers(msg):
     return msg
 
 
+# GLOBALLY_SUPPORTED_CHECKSUMS = ['adler32', 'md5', 'sha256', 'crc32']
+GLOBALLY_SUPPORTED_CHECKSUMS = ['adler32', 'md5']
+CHECKSUM_ALGO_DICT = {}
+PREFERRED_CHECKSUM = GLOBALLY_SUPPORTED_CHECKSUMS[0]
+CHECKSUM_KEY = 'supported_checksums'
+
+
+def is_checksum_valid(checksum_name):
+    """
+    A simple function to check wether a checksum algorithm is supported.
+    Relies on GLOBALLY_SUPPORTED_CHECKSUMS to allow for expandability.
+
+    :param checksum_name: The name of the checksum to be verified.
+    :returns: True if checksum_name is in GLOBALLY_SUPPORTED_CHECKSUMS list, False otherwise.
+    """
+
+    return checksum_name in GLOBALLY_SUPPORTED_CHECKSUMS
+
+
+def set_checksum_value(file, checksum_names_list):
+    for checksum_name in checksum_names_list:
+        if checksum_name in file['metadata'].keys() and file['metadata'][checksum_name]:
+            file['checksum'] = '%s:%s' % (checksum_name.upper(), str(file['metadata'][checksum_name]))
+            if checksum_name == PREFERRED_CHECKSUM:
+                break
+
+
 def adler32(file):
     """
     An Adler-32 checksum is obtained by calculating two 16-bit checksums A and B and concatenating their bits into a 32-bit integer. A is the sum of all bytes in the stream plus one, and B is the sum of the individual values of A from each step.
 
+    :param file: file name
     :returns: Hexified string, padded to 8 values.
     """
 
@@ -204,11 +320,14 @@ def adler32(file):
     return str('%08x' % adler)
 
 
+CHECKSUM_ALGO_DICT['adler32'] = adler32
+
+
 def md5(file):
     """
     Runs the MD5 algorithm (RFC-1321) on the binary content of the file named file and returns the hexadecimal digest
 
-    :param string: file name
+    :param file: file name
     :returns: string of 32 hexadecimal digits
     """
     hash_md5 = hashlib.md5()
@@ -221,6 +340,42 @@ def md5(file):
     return hash_md5.hexdigest()
 
 
+CHECKSUM_ALGO_DICT['md5'] = md5
+
+
+def sha256(file):
+    """
+    Runs the SHA256 algorithm on the binary content of the file named file and returns the hexadecimal digest
+
+    :param file: file name
+    :returns: string of 32 hexadecimal digits
+    """
+    with open(file, "rb") as f:
+        bytes = f.read()  # read entire file as bytes
+        readable_hash = hashlib.sha256(bytes).hexdigest()
+        print(readable_hash)
+        return readable_hash
+
+
+CHECKSUM_ALGO_DICT['sha256'] = sha256
+
+
+def crc32(file):
+    """
+    Runs the CRC32 algorithm on the binary content of the file named file and returns the hexadecimal digest
+
+    :param file: file name
+    :returns: string of 32 hexadecimal digits
+    """
+    prev = 0
+    for eachLine in open(file, "rb"):
+        prev = zlib.crc32(eachLine, prev)
+    return "%X" % (prev & 0xFFFFFFFF)
+
+
+CHECKSUM_ALGO_DICT['crc32'] = crc32
+
+
 def str_to_date(string):
     """ Converts a RFC-1123 string to the corresponding datetime value.
 
@@ -229,6 +384,21 @@ def str_to_date(string):
     return datetime.datetime.strptime(string, DATE_FORMAT) if string else None
 
 
+def val_to_space_sep_str(vallist):
+    """ Converts a list of values into a string of space separated values
+
+    :param vallist: the list of values to to convert into string
+    :return: the string of space separated values or the value initially passed as parameter
+    """
+    try:
+        if isinstance(vallist, list):
+            return u" ".join(vallist)
+        else:
+            return unicode(vallist)
+    except Exception:
+        return u''
+
+
 def date_to_str(date):
     """ Converts a datetime value to the corresponding RFC-1123 string.
 
@@ -493,15 +663,28 @@ def construct_surl_BelleII(dsn, filename):
         return '%s/%s' % (dsn, filename)
 
 
-def construct_surl(dsn, filename, naming_convention=None):
-    if naming_convention == 'T0':
-        return construct_surl_T0(dsn, filename)
-    elif naming_convention == 'DQ2':
-        return construct_surl_DQ2(dsn, filename)
-    elif naming_convention == 'BelleII':
-        return construct_surl_BelleII(dsn, filename)
+_SURL_ALGORITHMS = {}
+_DEFAULT_SURL = 'DQ2'
 
-    return construct_surl_DQ2(dsn, filename)
+
+def register_surl_algorithm(surl_callable, name=None):
+    if name is None:
+        name = surl_callable.__name__
+    _SURL_ALGORITHMS[name] = surl_callable
+
+
+register_surl_algorithm(construct_surl_T0, 'T0')
+register_surl_algorithm(construct_surl_DQ2, 'DQ2')
+register_surl_algorithm(construct_surl_BelleII, 'BelleII')
+
+
+def construct_surl(dsn, filename, naming_convention=None):
+    # ensure that policy package is loaded in case it registers its own algorithms
+    import rucio.common.schema  # noqa: F401
+
+    if naming_convention is None or naming_convention not in _SURL_ALGORITHMS:
+        naming_convention = _DEFAULT_SURL
+    return _SURL_ALGORITHMS[naming_convention](dsn, filename)
 
 
 def __strip_dsn(dsn):
@@ -545,14 +728,80 @@ def clean_surls(surls):
     for surl in surls:
         if surl.startswith('srm'):
             surl = re.sub(':[0-9]+/', '/', surl)
-            surl = re.sub('/srm/managerv1\?SFN=', '', surl)
-            surl = re.sub('/srm/v2/server\?SFN=', '', surl)
-            surl = re.sub('/srm/managerv2\?SFN=', '', surl)
+            surl = re.sub('/srm/managerv1\?SFN=', '', surl)  # NOQA: W605
+            surl = re.sub('/srm/v2/server\?SFN=', '', surl)  # NOQA: W605
+            surl = re.sub('/srm/managerv2\?SFN=', '', surl)  # NOQA: W605
         res.append(surl)
     res.sort()
     return res
 
 
+_EXTRACT_SCOPE_ALGORITHMS = {}
+_DEFAULT_EXTRACT = 'atlas'
+
+
+def extract_scope_atlas(did, scopes):
+    # Try to extract the scope from the DSN
+    if did.find(':') > -1:
+        if len(did.split(':')) > 2:
+            raise RucioException('Too many colons. Cannot extract scope and name')
+        scope, name = did.split(':')[0], did.split(':')[1]
+        if name.endswith('/'):
+            name = name[:-1]
+        return scope, name
+    else:
+        scope = did.split('.')[0]
+        if did.startswith('user') or did.startswith('group'):
+            scope = ".".join(did.split('.')[0:2])
+        if did.endswith('/'):
+            did = did[:-1]
+        return scope, did
+
+
+def extract_scope_belleii(did, scopes):
+    split_did = did.split('/')
+    if did.find('/belle/MC/') > -1:
+        if len(split_did) > 5:
+            if split_did[4] in ['fab', 'merge1', 'skim']:
+                return 'mc_tmp', did
+        return 'mc', did
+    if did.find('/belle/Raw/') > -1:
+        return 'raw', did
+    if did.find('/belle/user/') > -1:
+        if len(split_did) > 5:
+            if len(split_did[4]) == 1 and 'user.%s' % (split_did[5]) in scopes:
+                return 'user.%s' % split_did[5], did
+        if len(split_did) > 4:
+            if 'user.%s' % (split_did[4]) in scopes:
+                return 'user.%s' % split_did[4], did
+        return 'user', did
+    if did.find('/belle/data/') > -1:
+        if len(split_did) > 5:
+            if split_did[4] in ['fab', 'skim']:
+                return 'data_tmp', did
+        return 'data', did
+    if did.find('/belle/ddm/functional_tests/') > -1:
+        return 'test', did
+    return 'other', did
+
+
+def register_extract_scope_algorithm(extract_callable, name=[]):
+    if name is None:
+        name = extract_callable.__name__
+    _EXTRACT_SCOPE_ALGORITHMS[name] = extract_callable
+
+
+register_extract_scope_algorithm(extract_scope_atlas, 'atlas')
+register_extract_scope_algorithm(extract_scope_belleii, 'belleii')
+
+
+def extract_scope(did, scopes=None):
+    extract_scope_convention = config_get('common', 'extract_scope', False, None)
+    if extract_scope_convention is None or extract_scope_convention not in _EXTRACT_SCOPE_ALGORITHMS:
+        extract_scope_convention = _DEFAULT_EXTRACT
+    return _EXTRACT_SCOPE_ALGORITHMS[extract_scope_convention](did=did, scopes=scopes)
+
+
 def pid_exists(pid):
     """
     Check whether pid exists in the current process table.
@@ -637,7 +886,7 @@ def is_archive(name):
 
     :return: A boolean.
     '''
-    regexp = '^.*\.(zip|zipx|tar.gz|tgz|tar.Z|tar.bz2|tbz2)(\.\d+)*$'
+    regexp = r'^.*\.(zip|zipx|tar.gz|tgz|tar.Z|tar.bz2|tbz2)(\.\d+)*$'
     if re.match(regexp, name, re.I):
         return True
     return False
@@ -701,6 +950,8 @@ def ssh_sign(private_key, message):
     :param message: The message to sign as a string.
     :return: Base64 encoded signature as a string.
     """
+    if PY3 and isinstance(message, str):
+        message = message.encode()
     if not EXTRA_MODULES['paramiko']:
         raise MissingModuleException('The paramiko module is not installed or faulty.')
     sio_private_key = StringIO(private_key)
@@ -708,7 +959,10 @@ def ssh_sign(private_key, message):
     sio_private_key.close()
     signature_stream = priv_k.sign_ssh_data(message)
     signature_stream.rewind()
-    return base64.b64encode(signature_stream.get_remainder())
+    base64_encoded = base64.b64encode(signature_stream.get_remainder())
+    if PY3:
+        base64_encoded = base64_encoded.decode()
+    return base64_encoded
 
 
 def make_valid_did(lfn_dict):
@@ -1045,3 +1299,50 @@ def api_update_return_dict(dictionary):
         dictionary['scope'] = dictionary['scope'].external
 
     return dictionary
+
+
+def get_parsed_throttler_mode(throttler_mode):
+    """ Parse the conveyor-throttler mode string. """
+    direction = None
+    all_activities = None
+    if throttler_mode == 'DEST_PER_ACT':
+        direction = 'destination'
+        all_activities = False
+    elif throttler_mode == 'DEST_PER_ALL_ACT':
+        direction = 'destination'
+        all_activities = True
+    elif throttler_mode == 'SRC_PER_ACT':
+        direction = 'source'
+        all_activities = False
+    elif throttler_mode == 'SRC_PER_ALL_ACT':
+        direction = 'source'
+        all_activities = True
+    return (direction, all_activities)
+
+
+def query_bunches(query, bunch_by):
+    """
+    Queries output by yield_per sqlalchemy function
+    (which in a for loop returns rows one by one).
+    Groups the query rows in bunches of bunch_by
+    elements and returns list of bunches.
+    :param query: sqlalchemy session query
+    :param bunch_by: integer number
+    :returns: [[bunch_of_tuples_1],[bunch_of_tuples_2],...]
+
+    """
+    filtered_bunches = []
+    item_bunch = []
+    for i in query.yield_per(bunch_by):
+        # i is either tuple of one element (token/model object etc.)
+        if not isinstance(i, tuple) and not isinstance(i, list):
+            item_bunch.append(i)
+        # or i is a tuple with the column elements per row
+        else:
+            item_bunch += i
+        if len(item_bunch) % bunch_by == 0:
+            filtered_bunches.append(item_bunch)
+            item_bunch = []
+    if item_bunch:
+        filtered_bunches.append(item_bunch)
+    return filtered_bunches
