@@ -14,7 +14,7 @@
 #
 # Authors:
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2017-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2017-2020
 # - Vincent Garonne <vgaronne@gmail.com>, 2017
 # - Igor Mandrichenko <rucio@fermicloud055.fnal.gov>, 2018
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2018-2020
@@ -135,19 +135,20 @@ def submit_bulk_transfers(external_host, files, transfertool='fts3', job_params=
         use_oidc = job_params.get('use_oidc', False)
         transfer_token = None
         if use_oidc:
+            logging.debug('OAuth2/OIDC available at RSEs')
             account = job_params.get('account', None)
             getadmintoken = False
             if ALLOW_USER_OIDC_TOKENS is False:
                 getadmintoken = True
+            logging.debug('Attempting to get a token for account %s. Admin token option set to %s' % (account, getadmintoken))
             # find the appropriate OIDC token and exchange it (for user accounts) if necessary
-            token_object = get_token_for_account_operation(account, req_audience=REQUEST_OIDC_AUDIENCE, req_scope=REQUEST_OIDC_SCOPE, admin=getadmintoken)
-            if token_object is not None:
-                transfer_token = token_object.token
-        if not user_transfer_job:
-            transfer_id = FTS3Transfertool(external_host=external_host, token=transfer_token).submit(files=job_files, job_params=job_params, timeout=timeout)
-        else:
-            # if no valid USER TRANSFER cases --> go with std submission
-            transfer_id = FTS3Transfertool(external_host=external_host, token=transfer_token).submit(files=job_files, job_params=job_params, timeout=timeout)
+            token_dict = get_token_for_account_operation(account, req_audience=REQUEST_OIDC_AUDIENCE, req_scope=REQUEST_OIDC_SCOPE, admin=getadmintoken)
+            if token_dict is not None:
+                logging.debug('Access token has been granted.')
+                if 'token' in token_dict:
+                    logging.debug('Access token used as transfer token.')
+                    transfer_token = token_dict['token']
+        transfer_id = FTS3Transfertool(external_host=external_host, token=transfer_token).submit(files=job_files, job_params=job_params, timeout=timeout)
         record_timer('core.request.submit_transfers_fts3', (time.time() - start_time) * 1000 / len(files))
     elif transfertool == 'globus':
         logging.debug('... Starting globus xfer ...')
@@ -165,6 +166,9 @@ def submit_bulk_transfers(external_host, files, transfertool='fts3', job_params=
             job_files.append(job_file)
         logging.debug('job_files: %s' % job_files)
         transfer_id = GlobusTransferTool(external_host=None).bulk_submit(submitjob=job_files, timeout=timeout)
+    elif transfertool == 'mock':
+        import uuid
+        transfer_id = str(uuid.uuid1())
     return transfer_id
 
 
@@ -484,8 +488,7 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
     if multihop_rses is None:
         multihop_rses = []
 
-    # TODO: Might be problematic to always load the distance_graph, since it might be expensive
-    # TODO: Have an rse_expression to specify the eligible hops
+    # TODO: Might be problematic to always load the distance_graph, since it might be expensiv
 
     # Load the graph from the distances table
     # distance_graph = __load_distance_graph(session=session)
@@ -519,6 +522,11 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
     # 2. There is no connection or no scheme match --> Try a multi hop --> Dijkstra algorithm
     HOP_PENALTY = core_config_get('transfers', 'hop_penalty', default=10, session=session)  # Penalty to be applied to each further hop
 
+    # Check if the destination RSE is an island RSE:
+    if not __load_distance_edges_node(rse_id=dest_rse_id, session=session):
+        # The destination RSE is an island
+        raise NoDistance()
+
     visited_nodes = {source_rse_id: {'distance': 0,
                                      'path': []}}  # Dijkstra already visisted nodes
     # {rse_id: {'path': [{'source_rse_id':, 'dest_rse_id':, 'source_scheme', 'dest_scheme': }],
@@ -776,7 +784,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
 
                 # III - Extend the metadata dictionary with request attributes
                 overwrite, bring_online = True, None
-                if rses_info[source_rse_id]['rse_type'] == RSEType.TAPE or rses_info[source_rse_id]['rse_type'] == 'TAPE':
+                if rses_info[source_rse_id]['rse_type'] == RSEType.TAPE or rses_info[source_rse_id]['rse_type'] == 'TAPE' or rse_attrs[source_rse_id].get('staging_required', False):
                     bring_online = bring_online_local
                     transfer_src_type = "TAPE"
                     if not allow_tape_source:
@@ -800,9 +808,16 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     if source_protocol in ['davs', 'https']:
                         source_url += '?copy_mode=%s' % WEBDAV_TRANSFER_MODE
 
+                source_sign_url = rse_attrs[source_rse_id].get('sign_url', None)
+                if source_sign_url == 'gcs':
+                    source_url = re.sub('davs', 'gclouds', source_url)
+                    source_url = re.sub('https', 'gclouds', source_url)
+
                 # IV - get external_host + strict_copy
                 strict_copy = rse_attrs[dest_rse_id].get('strict_copy', False)
                 fts_hosts = rse_attrs[dest_rse_id].get('fts', None)
+                if source_sign_url == 'gcs':
+                    fts_hosts = rse_attrs[source_rse_id].get('fts', None)
                 source_globus_endpoint_id = rse_attrs[source_rse_id].get('globus_endpoint_id', None)
                 dest_globus_endpoint_id = rse_attrs[dest_rse_id].get('globus_endpoint_id', None)
 
@@ -814,10 +829,11 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     continue
                 if retry_count is None:
                     retry_count = 0
+                external_host = ''
                 if fts_hosts:
                     fts_list = fts_hosts.split(",")
+                    external_host = fts_list[0]
 
-                external_host = fts_list[0]
                 if retry_other_fts:
                     external_host = fts_list[retry_count % len(fts_list)]
 
@@ -945,6 +961,11 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     if source_protocol in ['davs', 'https']:
                         source_url += '?copy_mode=%s' % WEBDAV_TRANSFER_MODE
 
+                source_sign_url = rse_attrs[source_rse_id].get('sign_url', None)
+                if source_sign_url == 'gcs':
+                    source_url = re.sub('davs', 'gclouds', source_url)
+                    source_url = re.sub('https', 'gclouds', source_url)
+
                 # III - The transfer queued previously is a multihop, but this one is direct.
                 # Reset the sources, remove the multihop flag
                 if transfers[req_id].get('multihop', False):
@@ -956,7 +977,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                 # TAPE should not mixed with Disk and should not use as first try
                 # If there is a source whose ranking is no less than the Tape ranking, Tape will not be used.
 
-                if rses_info[source_rse_id]['rse_type'] == RSEType.TAPE or rses_info[source_rse_id]['rse_type'] == 'TAPE':
+                if rses_info[source_rse_id]['rse_type'] == RSEType.TAPE or rses_info[source_rse_id]['rse_type'] == 'TAPE' or rse_attrs[source_rse_id].get('staging_required', False):
                     # current src_rse is Tape
                     if not allow_tape_source:
                         continue
@@ -1102,7 +1123,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     new_req = queue_requests(requests=[{'dest_rse_id': dest_rse_id,
                                                         'scope': scope,
                                                         'name': name,
-                                                        'rule_id': None,
+                                                        'rule_id': '00000000000000000000000000000000',  # Dummy Rule ID used for multihop. TODO: Replace with actual rule_id once we can flag intermediate requests
                                                         'attributes': req_attributes,
                                                         'request_type': RequestType.TRANSFER,
                                                         'retry_count': retry_count,
@@ -1194,6 +1215,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     transfers[new_req_id] = {'request_id': new_req_id,
                                              'initial_request_id': req_id,
                                              'parent_request': parent_request,
+                                             'account': InternalAccount('root'),
                                              'schemes': __add_compatible_schemes(schemes=[destination_protocol], allowed_schemes=current_schemes),
                                              # 'src_urls': [source_url],
                                              'sources': [(source_rse_name, source_url, source_rse_id, 0, 0)],
