@@ -18,6 +18,8 @@
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -29,7 +31,7 @@ import socket
 import threading
 import time
 
-from sys import exc_info, stdout, argv
+from sys import exc_info, stdout
 from traceback import format_exception
 
 from rucio.db.sqla.constants import LifetimeExceptionsState
@@ -65,7 +67,7 @@ def atropos(thread, bulk, date_check, dry_run=True, grace_period=86400,
 
     sleep_time = 60
 
-    executable = ' '.join(argv)
+    executable = 'atropos'
     hostname = socket.getfqdn()
     pid = os.getpid()
     hb_thread = threading.current_thread()
@@ -74,7 +76,7 @@ def atropos(thread, bulk, date_check, dry_run=True, grace_period=86400,
     hb = heartbeat.live(executable, hostname, pid, hb_thread)
     time.sleep(10)
     hb = heartbeat.live(executable, hostname, pid, hb_thread)
-    prepend_str = 'Thread [%i/%i] : ' % (hb['assign_thread'] + 1, hb['nr_threads'])
+    prepend_str = 'Thread [%i/%i] : ' % (hb['assign_thread'], hb['nr_threads'])
     logging.debug(prepend_str + 'Starting worker')
     summary = {}
     lifetime_exceptions = {}
@@ -91,11 +93,11 @@ def atropos(thread, bulk, date_check, dry_run=True, grace_period=86400,
         while not GRACEFUL_STOP.is_set():
 
             hb = heartbeat.live(executable, hostname, pid, hb_thread)
-            prepend_str = 'Thread [%i/%i] : ' % (hb['assign_thread'] + 1, hb['nr_threads'])
+            prepend_str = 'Thread [%i/%i] : ' % (hb['assign_thread'], hb['nr_threads'])
 
             stime = time.time()
             try:
-                rules = get_rules_beyond_eol(date_check, thread, hb['nr_threads'] - 1, session=None)
+                rules = get_rules_beyond_eol(date_check, thread, hb['nr_threads'], session=None)
                 logging.info(prepend_str + '%s rules to process' % (len(rules)))
                 for rule_idx, rule in enumerate(rules, start=1):
                     did = '%s:%s' % (rule.scope, rule.name)
@@ -153,7 +155,7 @@ def atropos(thread, bulk, date_check, dry_run=True, grace_period=86400,
                             logging.info(prepend_str + 'Setting %s seconds lifetime for rule %s' % (lifetime, rule.id))
                             options = {'lifetime': lifetime}
                             if purge_replicas:
-                                options = {'purge_replicas': True}
+                                options['purge_replicas'] = True
                             if rule.locked and unlock:
                                 logging.info(prepend_str + 'Unlocking rule %s', rule.id)
                                 options['locked'] = False
