@@ -16,6 +16,8 @@
 # - Martin Barisits <martin.barisits@cern.ch>, 2015-2017
 # - Vincent Garonne <vgaronne@gmail.com>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -67,13 +69,14 @@ def rule_injector(once=False):
     paused_rules = {}  # {rule_id: datetime}
 
     # Make an initial heartbeat so that all judge-inectors have the correct worker number on the next try
-    live(executable='rucio-judge-injector', hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
+    executable = 'judge-injector'
+    live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
     graceful_stop.wait(1)
 
     while not graceful_stop.is_set():
         try:
             # heartbeat
-            heartbeat = live(executable='rucio-judge-injector', hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
+            heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
 
             start = time.time()
 
@@ -83,30 +86,30 @@ def rule_injector(once=False):
                 if datetime.utcnow() > paused_rules[key]:
                     del paused_rules[key]
 
-            rules = get_injected_rules(total_workers=heartbeat['nr_threads'] - 1,
+            rules = get_injected_rules(total_workers=heartbeat['nr_threads'],
                                        worker_number=heartbeat['assign_thread'],
                                        limit=100,
                                        blacklisted_rules=[key for key in paused_rules])
-            logging.debug('rule_injector[%s/%s] index query time %f fetch size is %d' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, time.time() - start, len(rules)))
+            logging.debug('rule_injector[%s/%s] index query time %f fetch size is %d' % (heartbeat['assign_thread'], heartbeat['nr_threads'], time.time() - start, len(rules)))
 
             if not rules and not once:
-                logging.debug('rule_injector[%s/%s] did not get any work (paused_rules=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, str(len(paused_rules))))
+                logging.debug('rule_injector[%s/%s] did not get any work (paused_rules=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'], str(len(paused_rules))))
                 graceful_stop.wait(60)
             else:
                 for rule in rules:
                     rule_id = rule[0]
-                    logging.info('rule_injector[%s/%s]: Injecting rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, rule_id))
+                    logging.info('rule_injector[%s/%s]: Injecting rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
                     if graceful_stop.is_set():
                         break
                     try:
                         start = time.time()
                         inject_rule(rule_id=rule_id)
-                        logging.debug('rule_injector[%s/%s]: injection of %s took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, rule_id, time.time() - start))
+                        logging.debug('rule_injector[%s/%s]: injection of %s took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id, time.time() - start))
                     except (DatabaseException, DatabaseError) as e:
                         if match('.*ORA-00054.*', str(e.args[0])):
                             paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
                             record_counter('rule.judge.exceptions.LocksDetected')
-                            logging.warning('rule_injector[%s/%s]: Locks detected for %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, rule_id))
+                            logging.warning('rule_injector[%s/%s]: Locks detected for %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
                         elif match('.*QueuePool.*', str(e.args[0])):
                             logging.warning(traceback.format_exc())
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
@@ -118,18 +121,18 @@ def rule_injector(once=False):
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except RSEBlacklisted as e:
                         paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
-                        logging.warning('rule_injector[%s/%s]: RSEBlacklisted for rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, rule_id))
+                        logging.warning('rule_injector[%s/%s]: RSEBlacklisted for rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except ReplicationRuleCreationTemporaryFailed as e:
                         paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
-                        logging.warning('rule_injector[%s/%s]: ReplicationRuleCreationTemporaryFailed for rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, rule_id))
+                        logging.warning('rule_injector[%s/%s]: ReplicationRuleCreationTemporaryFailed for rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except RuleNotFound as e:
                         pass
                     except InsufficientAccountLimit as e:
                         # A rule with InsufficientAccountLimit on injection hangs there potentially forever
                         # It should be marked as SUSPENDED
-                        logging.info('rule_injector[%s/%s]: Marking rule %s as SUSPENDED due to InsufficientAccountLimit' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, rule_id))
+                        logging.info('rule_injector[%s/%s]: Marking rule %s as SUSPENDED due to InsufficientAccountLimit' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
                         update_rule(rule_id=rule_id, options={'state': 'SUSPENDED'})
 
         except (DatabaseException, DatabaseError) as e:
@@ -148,7 +151,7 @@ def rule_injector(once=False):
         if once:
             break
 
-    die(executable='rucio-judge-injector', hostname=hostname, pid=pid, thread=current_thread)
+    die(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
 
 
 def stop(signum=None, frame=None):
@@ -164,8 +167,9 @@ def run(once=False, threads=1):
     Starts up the Judge-Injector threads.
     """
 
+    executable = 'judge-injector'
     hostname = socket.gethostname()
-    sanity_check(executable='rucio-judge-injector', hostname=hostname)
+    sanity_check(executable=executable, hostname=hostname)
 
     if once:
         rule_injector(once)
