@@ -348,7 +348,7 @@ def reaper(rses, include_rses, exclude_rses, chunk_size=100, once=False, greedy=
     :param sleep_time:     Time between two cycles.
     """
     hostname = socket.getfqdn()
-    executable = sys.argv[0]
+    executable = 'reaper2'
     pid = os.getpid()
     hb_thread = threading.current_thread()
     sanity_check(executable=executable, hostname=hostname)
@@ -504,9 +504,9 @@ def reaper(rses, include_rses, exclude_rses, chunk_size=100, once=False, greedy=
                         del_start_time = time.time()
                         for replica in file_replicas:
                             try:
-                                replica['pfn'] = str(rsemgr.lfns2pfns(rse_settings=rse_info,
-                                                                      lfns=[{'scope': replica['scope'], 'name': replica['name'], 'path': replica['path']}],
-                                                                      operation='delete', scheme=scheme).values()[0])
+                                replica['pfn'] = str(list(rsemgr.lfns2pfns(rse_settings=rse_info,
+                                                                           lfns=[{'scope': replica['scope'], 'name': replica['name'], 'path': replica['path']}],
+                                                                           operation='delete', scheme=scheme).values())[0])
                             except (ReplicaUnAvailable, ReplicaNotFound) as error:
                                 logging.warning('%s Failed get pfn UNAVAILABLE replica %s:%s on %s with error %s', prepend_str, replica['scope'], replica['name'], rse_name, str(error))
                                 replica['pfn'] = None
