@@ -13,7 +13,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2020
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2014
 # - Vincent Garonne <vgaronne@gmail.com>, 2015-2018
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2015
@@ -47,7 +47,7 @@ from traceback import format_exc
 from stomp import Connection
 
 from rucio.common.config import config_get, config_get_bool, config_get_int
-from rucio.common.exception import ConfigNotFound
+from rucio.common.exception import ConfigNotFound, RSENotFound
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.core.monitor import record_counter, record_timer
 from rucio.core.config import get
@@ -217,7 +217,12 @@ class AMQConsumer(object):
 
                     rses = report['remoteSite'].strip().split(',')
                     for rse in rses:
-                        rse_id = get_rse_id(rse=rse)
+                        try:
+                            rse_id = get_rse_id(rse=rse)
+                        except RSENotFound:
+                            logging.error(format_exc())
+                            record_counter('daemons.tracer.kronos.rse_not_found')
+                            continue
                         replicas.append({'name': report['filename'], 'scope': report['scope'], 'rse': rse, 'rse_id': rse_id, 'accessed_at': datetime.utcfromtimestamp(report['traceTimeentryUnix']),
                                          'traceTimeentryUnix': report['traceTimeentryUnix'], 'eventVersion': report['eventVersion']})
                 else:
@@ -228,9 +233,13 @@ class AMQConsumer(object):
                     rse = None
                     if 'remoteSite' in report:
                         rse = report['remoteSite']
-                        rse_id = get_rse_id(rse=rse)
+                        try:
+                            rse_id = get_rse_id(rse=rse)
+                        except RSENotFound:
+                            logging.error(format_exc())
+                            record_counter('daemons.tracer.kronos.rse_not_found')
                     if 'datasetScope' in report:
-                        self.__dataset_queue.put({'scope': report['datasetScope'], 'name': report['dataset'], 'rse_id': rse_id, 'accessed_at': datetime.utcfromtimestamp(report['traceTimeentryUnix'])})
+                        self.__dataset_queue.put({'scope': InternalScope(report['datasetScope']), 'name': report['dataset'], 'rse_id': rse_id, 'accessed_at': datetime.utcfromtimestamp(report['traceTimeentryUnix'])})
                         continue
                     else:
                         if 'remoteSite' not in report:
@@ -249,7 +258,12 @@ class AMQConsumer(object):
                 if did['scope'].external == 'panda' and '_dis' in did['name']:
                     continue
                 for rse in rses:
-                    rse_id = get_rse_id(rse=rse)
+                    try:
+                        rse_id = get_rse_id(rse=rse)
+                    except RSENotFound:
+                        logging.error(format_exc())
+                        record_counter('daemons.tracer.kronos.rse_not_found')
+                        continue
                     self.__dataset_queue.put({'scope': did['scope'], 'name': did['name'], 'did_type': did['type'], 'rse_id': rse_id, 'accessed_at': datetime.utcfromtimestamp(report['traceTimeentryUnix'])})
 
         logging.debug(replicas)
@@ -279,6 +293,7 @@ def kronos_file(once=False, thread=0, brokers_resolved=None, dataset_queue=None,
 
     logging.info('tracer consumer starting')
 
+    executable = 'kronos-file'
     hostname = socket.gethostname()
     pid = getpid()
     thread = current_thread()
@@ -329,10 +344,10 @@ def kronos_file(once=False, thread=0, brokers_resolved=None, dataset_queue=None,
 
     logging.info('(kronos_file) tracer consumer started')
 
-    sanity_check(executable='kronos-file', hostname=hostname)
+    sanity_check(executable=executable, hostname=hostname)
     while not graceful_stop.is_set():
         start_time = time()
-        live(executable='kronos-file', hostname=hostname, pid=pid, thread=thread)
+        live(executable=executable, hostname=hostname, pid=pid, thread=thread)
         for conn in conns:
             if not conn.is_connected():
                 logging.info('(kronos_file) connecting to %s' % conn.transport._Transport__host_and_ports[0][0])
@@ -364,23 +379,24 @@ def kronos_file(once=False, thread=0, brokers_resolved=None, dataset_queue=None,
         except Exception:
             pass
 
-    die(executable='kronos-file', hostname=hostname, pid=pid, thread=thread)
+    die(executable=executable, hostname=hostname, pid=pid, thread=thread)
     logging.info('(kronos_file) graceful stop done')
 
 
 def kronos_dataset(once=False, thread=0, dataset_queue=None, sleep_time=60):
     logging.info('(kronos_dataset) starting')
 
+    executable = 'kronos-dataset'
     hostname = socket.gethostname()
     pid = getpid()
     thread = current_thread()
 
     dataset_wait = config_get_int('tracer-kronos', 'dataset_wait')
     start = datetime.now()
-    sanity_check(executable='kronos-dataset', hostname=hostname)
+    sanity_check(executable=executable, hostname=hostname)
     while not graceful_stop.is_set():
         start_time = time()
-        live(executable='kronos-dataset', hostname=hostname, pid=pid, thread=thread)
+        live(executable=executable, hostname=hostname, pid=pid, thread=thread)
         if (datetime.now() - start).seconds > dataset_wait:
             __update_datasets(dataset_queue)
             start = datetime.now()
@@ -389,7 +405,7 @@ def kronos_dataset(once=False, thread=0, dataset_queue=None, sleep_time=60):
             logging.info('(kronos_dataset) Will sleep for %s seconds' % (sleep_time - tottime))
             sleep(sleep_time - tottime)
     # once again for the backlog
-    die(executable='kronos-dataset', hostname=hostname, pid=pid, thread=thread)
+    die(executable=executable, hostname=hostname, pid=pid, thread=thread)
     logging.info('(kronos_dataset) cleaning dataset backlog before shutdown...')
     __update_datasets(dataset_queue)
 
