@@ -18,16 +18,22 @@
 # - Mario Lassnig <mario@lassnig.net>, 2018-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
 #
 # PY3K COMPATIBLE
 
 from __future__ import print_function
+from base64 import b64encode
 from datetime import datetime
+from hashlib import sha256
+from os import urandom
 from traceback import format_exc
 
 from alembic import command
 from alembic.config import Config
 
+from six import PY3
+
 from sqlalchemy import func
 from sqlalchemy.engine import reflection
 from sqlalchemy.schema import CreateSchema, MetaData, Table, DropTable, ForeignKeyConstraint, DropConstraint
@@ -72,7 +78,7 @@ def destroy_database(echo=True):
     try:
         models.unregister_models(engine)
     except Exception as e:
-            print('Cannot destroy schema -- assuming already gone, continuing:', e)
+        print('Cannot destroy schema -- assuming already gone, continuing:', e)
 
 
 def drop_everything(echo=True):
@@ -124,11 +130,22 @@ def drop_everything(echo=True):
     trans.commit()
 
 
+def create_base_vo():
+    """ Creates the base VO """
+
+    s = session.get_session()
+
+    vo = models.VO(vo='def', description='Default base VO', email='N/A')
+
+    s.add_all([vo])
+    s.commit()
+
+
 def create_root_account():
     """ Inserts the default root account to an existing database. Make sure to change the default password later. """
 
     up_id = 'ddmlab'
-    up_pwd = '2ccee6f6dd1bc2269cddd7cd5e47578e98e430539807c36df23fab7dd13e7583'
+    up_pwd = 'secret'
     up_email = 'ph-adp-ddm-lab@cern.ch'
     x509_id = '/C=CH/ST=Geneva/O=CERN/OU=PH-ADP-CO/CN=DDMLAB Client Certificate/emailAddress=ph-adp-ddm-lab@cern.ch'
     x509_email = 'ph-adp-ddm-lab@cern.ch'
@@ -159,7 +176,14 @@ def create_root_account():
 
     account = models.Account(account=InternalAccount('root'), account_type=AccountType.SERVICE, status=AccountStatus.ACTIVE)
 
-    identity1 = models.Identity(identity=up_id, identity_type=IdentityType.USERPASS, password=up_pwd, salt='0', email=up_email)
+    salt = urandom(255)
+    if PY3:
+        decoded_salt = b64encode(salt).decode()
+        salted_password = ('%s%s' % (decoded_salt, up_pwd)).encode()
+    else:
+        salted_password = '%s%s' % (salt, str(up_pwd))
+    hashed_password = sha256(salted_password).hexdigest()
+    identity1 = models.Identity(identity=up_id, identity_type=IdentityType.USERPASS, password=hashed_password, salt=salt, email=up_email)
     iaa1 = models.IdentityAccountAssociation(identity=identity1.identity, identity_type=identity1.identity_type, account=account.account, is_default=True)
 
     # X509 authentication
@@ -206,3 +230,15 @@ def get_db_time():
 
     finally:
         s.remove()
+
+
+def get_count(q):
+    """
+    Fast way to get count in SQLAlchemy
+    Source: https://gist.github.com/hest/8798884
+    Some limits, see a more thorough version above
+    """
+
+    count_q = q.statement.with_only_columns([func.count()]).order_by(None)
+    count = q.session.execute(count_q).scalar()
+    return count
