@@ -17,11 +17,12 @@
 # - Mario Lassnig, <mario.lassnig@cern.ch>, 2014-2019
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2019
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
 
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid as uuid
 from rucio.core.account import get_usage
-from rucio.core.account_limit import set_account_limit
+from rucio.core.account_limit import set_local_account_limit
 from rucio.core.did import add_did, attach_dids, detach_dids
 from rucio.core.lock import get_replica_locks, get_dataset_locks
 from rucio.core.rse import add_rse_attribute, get_rse_id
@@ -64,15 +65,15 @@ class TestJudgeEvaluator():
         # Add quota
         cls.jdoe = InternalAccount('jdoe')
         cls.root = InternalAccount('root')
-        set_account_limit(cls.jdoe, cls.rse1_id, -1)
-        set_account_limit(cls.jdoe, cls.rse3_id, -1)
-        set_account_limit(cls.jdoe, cls.rse4_id, -1)
-        set_account_limit(cls.jdoe, cls.rse5_id, -1)
+        set_local_account_limit(cls.jdoe, cls.rse1_id, -1)
+        set_local_account_limit(cls.jdoe, cls.rse3_id, -1)
+        set_local_account_limit(cls.jdoe, cls.rse4_id, -1)
+        set_local_account_limit(cls.jdoe, cls.rse5_id, -1)
 
-        set_account_limit(cls.root, cls.rse1_id, -1)
-        set_account_limit(cls.root, cls.rse3_id, -1)
-        set_account_limit(cls.root, cls.rse4_id, -1)
-        set_account_limit(cls.root, cls.rse5_id, -1)
+        set_local_account_limit(cls.root, cls.rse1_id, -1)
+        set_local_account_limit(cls.root, cls.rse3_id, -1)
+        set_local_account_limit(cls.root, cls.rse4_id, -1)
+        set_local_account_limit(cls.root, cls.rse5_id, -1)
 
     def test_judge_add_files_to_dataset(self):
         """ JUDGE EVALUATOR: Test the judge when adding files to dataset"""
