@@ -21,6 +21,7 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2018
 # - Martin Barisits <martin.barisits@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Luc Goossens <luc.goossens@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -205,7 +206,8 @@ class Replicas(RucioController):
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            add_replicas(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get('issuer'), ignore_availability=parameters.get('ignore_availability', False))
+            add_replicas(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get(
+                'issuer'), ignore_availability=parameters.get('ignore_availability', False))
         except InvalidPath as error:
             raise generate_http_error(400, 'InvalidPath', error.args[0])
         except AccessDenied as error:
@@ -274,7 +276,8 @@ class Replicas(RucioController):
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            delete_replicas(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get('issuer'), ignore_availability=parameters.get('ignore_availability', False))
+            delete_replicas(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get(
+                'issuer'), ignore_availability=parameters.get('ignore_availability', False))
         except AccessDenied as error:
             raise generate_http_error(401, 'AccessDenied', error.args[0])
         except RSENotFound as error:
@@ -322,6 +325,7 @@ class ListReplicas(RucioController):
         dids, schemes, select, unavailable, limit = [], None, None, False, None
         ignore_availability, rse_expression, all_states, domain = False, None, False, None
         signature_lifetime, resolve_archives, resolve_parents = None, True, False
+        updated_after = None
         client_location = {}
 
         json_data = data()
@@ -349,11 +353,21 @@ class ListReplicas(RucioController):
                 resolve_archives = params['resolve_archives']
             if 'resolve_parents' in params:
                 resolve_parents = params['resolve_parents']
+
             if 'signature_lifetime' in params:
                 signature_lifetime = params['signature_lifetime']
             else:
                 # hardcoded default of 10 minutes if config is not parseable
                 signature_lifetime = config_get('credentials', 'signature_lifetime', raise_exception=False, default=600)
+
+            if 'updated_after' in params:
+                if isinstance(params['updated_after'], (int, float)):
+                    # convert from epoch time stamp to datetime object
+                    updated_after = datetime.utcfromtimestamp(params['updated_after'])
+                else:
+                    # attempt UTC format '%Y-%m-%dT%H:%M:%S' conversion
+                    updated_after = datetime.strptime(params['updated_after'], '%Y-%m-%dT%H:%M:%S')
+
         except ValueError:
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
@@ -387,6 +401,7 @@ class ListReplicas(RucioController):
                                        domain=domain, signature_lifetime=signature_lifetime,
                                        resolve_archives=resolve_archives,
                                        resolve_parents=resolve_parents,
+                                       updated_after=updated_after,
                                        issuer=ctx.env.get('issuer')):
 
                 # in first round, set the appropriate content type, and stream the header
@@ -677,7 +692,8 @@ class BadReplicasStates(RucioController):
                 list_pfns = bool(params['list_pfns'][0])
 
         try:
-            result = list_bad_replicas_status(state=state, rse=rse, younger_than=younger_than, older_than=older_than, limit=limit, list_pfns=list_pfns)
+            result = list_bad_replicas_status(state=state, rse=rse, younger_than=younger_than,
+                                              older_than=older_than, limit=limit, list_pfns=list_pfns)
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -782,7 +798,7 @@ class DatasetReplicasVP(RucioController):
             406 Not Acceptable
             500 InternalError
 
-        :returns: If VP exists a list of dicts of sites, otherwise a list of dicts of dataset replicas
+        :returns: If VP exists a list of dicts of sites, otherwise nothing
         """
 
         header('Content-Type', 'application/x-json-stream')
